/*
	PlastererApplet
	Copyright (C) 2004 ONO, Masayuki <mono101@skill-engineering.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "plastererimpl.h"
#include "plastererdefine.h"

CPlastererImpl::CPlastererImpl( QWidget* parent )
	: QWidget( parent, NULL )
{
	// makeImages();
	setWallPaper();

	m_pChannel = new QCopChannel( "QPE/Plasterer", this );
	connect( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	m_pSysChannel = new QCopChannel( "QPE/System", this );
	connect( m_pSysChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveSysMessage( const QCString&, const QByteArray& ) ) );

	setFixedSize( 0, 0 );
}

CPlastererImpl::~CPlastererImpl()
{
	disconnect ( m_pChannel, SIGNAL( received( const QCString&, const QByteArray& ) ),
			this, SLOT( receiveMessage( const QCString&, const QByteArray& ) ) );
	delete m_pChannel;
	delete m_pSysChannel;
}

void CPlastererImpl::receiveMessage( const QCString& msg, const QByteArray& /*data*/ )
{
	// QDataStream stream( data, IO_ReadOnly );

	if ( msg == "version()")
	{
		version();
	}
	else if ( msg == "makeImages()" )
	{
		DBG_LOG( "CPlasterer::receiveMessage() > \"makeImages()\"" );
		makeImages();
	}
	else if ( msg == "reload()" )
	{
		DBG_LOG( "CPlasterer::receiveMessage() > \"reload()\"" );
		makeImages();
		setWallPaper();
	}
}

void CPlastererImpl::receiveLauncherMessage( const QCString& msg, const QByteArray& /*data*/ )
{
	DBG_LOG( QString( "CPlastererImpl::receiveLauncherMessage() > msg : %1" ).arg( msg ).latin1() );
}

void CPlastererImpl::receiveSysMessage( const QCString& msg, const QByteArray& data )
{
	if ( msg == "rotateChanged(QString)" )
	{
		QDataStream stream( data, IO_ReadOnly );
		QString rotInf;
		stream >> rotInf;
		DBG_LOG( QString( "CPlastererImpl::receiveSysMessage() > data : %1" ).arg( rotInf ).latin1() );
		setWallPaper( ( rotInf == "Transformed:Rot270:Vga:0" ) ? WF_H_IMG : WF_V_IMG );
	}
}

void CPlastererImpl::version()
{
	QString ver = "Plasterer";
	ver.append( _version_ );
	Global::statusMessage( ver );
}

void CPlastererImpl::paintEvent( QPaintEvent * )
{
	// QPainter p(this);
	// m_rData.draw( &p );
}

void CPlastererImpl::setWallPaper( const QString& sImg )
{
	QString sImgPath = QPEApplication::documentDir() + WF_PATH;
	if ( sImg != QString::null )
	{
		sImgPath += sImg;
	}
	else
	{
		sImgPath += SlMisc::isLandscape() ? WF_H_IMG : WF_V_IMG;
	}
	Config cfg( CNF_L );
	cfg.setGroup( SEC_WP );
	cfg.writeEntry( KEY_WFN, sImgPath );
	DBG_LOG( QString( "CPlastererImpl::setWallPaper() > ImgPath : %1" ).arg( sImgPath ).latin1() );
	
	{
		QCopEnvelope* pE = new QCopEnvelope( "QPE/System", "changedWallPaper()" );
		delete pE;
	}
}

void CPlastererImpl::makeImages()
{
	DBG_LOG( "CPlastererImpl::makeImages()" );

	QMessageBox mb( "PlastererApplet",
					"Making Images...",
					QMessageBox::Information,
					QMessageBox::NoButton,
					QMessageBox::NoButton,
					QMessageBox::NoButton );
	mb.showMaximized();
	
	QString sImgPath = QPEApplication::documentDir() + WF_PATH;
	Config cfg( CNF_P );
	cfg.setGroup( SEC_WP );

	QImage imgV( cfg.readEntry( KEY_VFN, "" ) );
	if ( ! cfg.readBoolEntry( KEY_VTM, TRUE ) && imgV.size() != QSize( 0, 0 ) )
	{
		DBG_LOG( "CPlastererImpl::makeImages() > tilingModeV is FALSE" );
		QSize rSizeDesktop = mb.size(); // QApplication::desktop()->size();
		QSize rSize;
		if ( rSizeDesktop.height() > rSizeDesktop.width() )
		{
			rSize = rSizeDesktop;
		}
		else
		{
			rSize = QSize( rSizeDesktop.height(), rSizeDesktop.width() );
		}
		DBG_LOG( QString( "CPlastererImpl::makeImages() > Desktop w:%1, h:%2" )
											.arg( rSize.width() )
											.arg( rSize.height() ).latin1() );
		QSize sizeScaling = imgV.size();
		DBG_LOG( QString( "CPlastererImpl::makeImages() > imgV w:%1, h:%2" )
											.arg( sizeScaling.width() )
											.arg( sizeScaling.height() ).latin1() );

		if ( cfg.readBoolEntry( KEY_VFT, FALSE ) )
		{
			DBG_LOG( "CPlastererImpl::makeImages() > fitToDispV is TRUE > smoothScale" );
			sizeScaling.setWidth( (double)sizeScaling.width() 
							* ( (double)rSize.height() / (double)sizeScaling.height() ) );
			sizeScaling.setHeight( rSize.height() );
			DBG_LOG( QString( "CPlastererImpl::makeImages() > scaled by height imgV w:%1, h:%2" )
												.arg( sizeScaling.width() )
												.arg( sizeScaling.height() ).latin1() );

			sizeScaling.setHeight( (double)sizeScaling.height() 
							* ( (double)rSize.width() / (double)sizeScaling.width() ) );
			sizeScaling.setWidth( rSize.width() );
			DBG_LOG( QString( "CPlastererImpl::makeImages() > scaled by width imgV w:%1, h:%2" )
												.arg( sizeScaling.width() )
												.arg( sizeScaling.height() ).latin1() );

			if ( imgV.size() != sizeScaling )
			{
				imgV = imgV.smoothScale( sizeScaling.width(), sizeScaling.height() );
				DBG_LOG( QString( "CPlastererImpl::makeImages() > scaled imgV w:%1, h:%2" )
													.arg( sizeScaling.width() )
													.arg( sizeScaling.height() ).latin1() );
			}
		}
		{
			DBG_LOG( "CPlastererImpl::makeImages() > CENTER ALIGN" );
			QPoint drawP( ( rSize.width() - sizeScaling.width() ) / 2,
							( rSize.height() - sizeScaling.height() ) / 2 );
			QPixmap pixT;
			pixT.resize( rSize );
			QStringList tmpList = QStringList::split( ",", cfg.readEntry( KEY_VBC, "0,0,0" ) );
			QColor tmpColor( tmpList[0].toInt(), tmpList[1].toInt(), tmpList[2].toInt() );
			pixT.fill( tmpColor );
			QPainter p( &pixT );
			p.drawImage( drawP, imgV );
			p.flush();
			imgV = pixT.convertToImage();
		}
	}
	imgV.save( sImgPath + WF_V_IMG, "PNG" );

	QImage imgH( cfg.readEntry( KEY_HFN, "" ) );
	if ( ! cfg.readBoolEntry( KEY_HTM, TRUE ) && imgH.size() != QSize( 0, 0 ) )
	{
		DBG_LOG( "CPlastererImpl::makeImages() > tilingModeH is FALSE" );
		QSize rSizeDesktop = mb.size(); // QApplication::desktop()->size();
		QSize rSize;
		if ( rSizeDesktop.height() > rSizeDesktop.width() )
		{
			rSize = QSize( rSizeDesktop.height(), rSizeDesktop.width() );
		}
		else
		{
			rSize = rSizeDesktop;
		}
		DBG_LOG( QString( "CPlastererImpl::makeImages() > Desktop w:%1, h:%2" )
											.arg( rSize.width() )
											.arg( rSize.height() ).latin1() );
		QSize sizeScaling = imgH.size();
		DBG_LOG( QString( "CPlastererImpl::makeImages() > imgH w:%1, h:%2" )
											.arg( sizeScaling.width() )
											.arg( sizeScaling.height() ).latin1() );

		if ( cfg.readBoolEntry( KEY_HFT, FALSE ) )
		{
			DBG_LOG( "CPlastererImpl::makeImages() > fitToDispH is TRUE > smoothScale" );
			sizeScaling.setHeight( (double)sizeScaling.height() 
							* ( (double)rSize.width() / (double)sizeScaling.width() ) );
			sizeScaling.setWidth( rSize.width() );
			DBG_LOG( QString( "CPlastererImpl::makeImages() > scaled by width imgH w:%1, h:%2" )
												.arg( sizeScaling.width() )
												.arg( sizeScaling.height() ).latin1() );

			sizeScaling.setWidth( (double)sizeScaling.width() 
							* ( (double)rSize.height() / (double)sizeScaling.height() ) );
			sizeScaling.setHeight( rSize.height() );
			DBG_LOG( QString( "CPlastererImpl::makeImages() > scaled by height imgH w:%1, h:%2" )
												.arg( sizeScaling.width() )
												.arg( sizeScaling.height() ).latin1() );

			if ( imgH.size() != sizeScaling )
			{
				imgH = imgH.smoothScale( sizeScaling.width(), sizeScaling.height() );
				DBG_LOG( QString( "CPlastererImpl::makeImages() > scaled imgH w:%1, h:%2" )
													.arg( sizeScaling.width() )
													.arg( sizeScaling.height() ).latin1() );
			}
		}
		{
			DBG_LOG( "CPlastererImpl::makeImages() > CENTER ALIGN" );
			QPoint drawP( ( rSize.width() - sizeScaling.width() ) / 2,
							( rSize.height() - sizeScaling.height() ) / 2 );
			QPixmap pixT;
			pixT.resize( rSize );
			QStringList tmpList = QStringList::split( ",", cfg.readEntry( KEY_HBC, "0,0,0" ) );
			QColor tmpColor( tmpList[0].toInt(), tmpList[1].toInt(), tmpList[2].toInt() );
			pixT.fill( tmpColor );
			QPainter p( &pixT );
			p.drawImage( drawP, imgH );
			p.flush();
			imgH = pixT.convertToImage();
		}
	}
	imgH.save( sImgPath + WF_H_IMG, "PNG" );
}


